# ---------------------------------------------------------------------------
# BadLinkFinder - A Plugin for Movable Type
#
# Release 0.4
# February 25, 2008
#
# Author: nlog(n)
# Description: http://nlogn.ath.cx/plugins/badlinkfinder/
# Email: n [at] nlogn [dot] ath [dot] cx
# ---------------------------------------------------------------------------

package MT::Plugin::BadLinkFinder;
use strict;
use MT::ConfigMgr;
use MT::Template::Context;
use Encode qw(encode decode);
MT::Template::Context->add_container_tag(BadLinkFinder => \&BadLinkFinder);

our $Name = 'BadLinkFinder';
our $VERSION = 0.4;
our $Exclude = '';

if (MT->version_number >= 3) {
  require MT::Plugin;
  my $plugin = MT::Plugin->new({
    name => "$Name $VERSION",
    description => 'Adds a class named &quot;badlink&quot; to the anchor tag when its reference URL returns error status code.',
    doc_link => 'http://nlogn.ath.cx/plugins/badlinkfinder/'
  });
  MT->add_plugin($plugin);
}

sub BadLinkFinder {
  my ($ctx, $args) = @_;

  my $tokens = $ctx->stash('tokens');
  my $builder = $ctx->stash('builder');

  defined(my $out = $builder->build($ctx, $tokens))
    or return $ctx->error($builder->errstr);

  return $out if (defined($ctx->stash('entry')) && $ctx->stash('entry')->to_ping_urls);

  my $charset = {
       'UTF-8' => 'utf8',
       'EUC-JP' => 'euc-jp',
       'Shift_JIS' => 'shiftjis',
       'ISO-2022-JP' => 'iso-2022-jp'
     }->{MT::ConfigMgr->instance->PublishCharset} || 'utf8';

  $Exclude ||= $ctx->stash('blog')->site_url;
  $out = decode($charset, $out);
  my $p = LinkChecker->new;
  $p->attr_encoded(1);
  $p->parse($out);
  $p->eof;
  $out = encode($charset, $p->filtered_html);
  $out;
}

1;

package LinkChecker;
use strict;
use LWP::UserAgent;
require HTML::Filter;
our @ISA = qw(HTML::Filter);

sub start {
  my($self, $tag, $attr, $attrseq, $orig) = @_;
  my @excludes = split /\|/, $Exclude;
  if ($tag eq 'a'
      && exists $attr->{href}
      && $attr->{href} =~ m|^http://| 
      && not grep {$attr->{href} =~ m|$_|} @excludes) {
    my $ua = LWP::UserAgent->new;
    $ua->agent(__PACKAGE__ . '/' . $VERSION);
    $ua->parse_head(0);
    $ua->timeout(5);
    my $req = HTTP::Request->new(HEAD => $attr->{href});
    my $res = $ua->request($req);
    my $flag = 0;
    my $tmp = "<$tag";
    for (@$attrseq) {
      if ($_ eq 'class' && $attr->{$_} !~ /badlink/ && $res->is_error) {
        $tmp .= qq( $_="$attr->{$_} badlink");
        $flag = 1;
      } elsif ($_ eq 'class' && $attr->{$_} =~ /badlink/) {
        $tmp .= qq( $_="$attr->{$_}");
        $flag = 1;
      } else {
        $tmp .= qq( $_="$attr->{$_}");
      }
    }
    $tmp .= qq( class="badlink") if $flag == 0 && $res->is_error;
    $tmp .= '>';
    $self->output($tmp);
    return;
  }
  $self->output($orig);
}

sub output { push(@{$_[0]->{fhtml}}, $_[1]) }
sub filtered_html { join('', @{$_[0]->{fhtml}}) }

1;
